@@ -8,21 +8,26 @@ from GoogleNews import GoogleNews
 from newspaper import Article, ArticleException
 from tqdm import tqdm, trange
 from threading import Thread
-from datetime import datetime, timedelta
+from datetime import date, datetime, timedelta
 import re
 import pandas as pd
 import time, requests
 import warnings
-import sys
+import sys, logging
+import twint
+
 from dateutil.parser._parser import UnknownTimezoneWarning
 import flair
 
+logging.getLogger("flair").setLevel(logging.WARNING)
+
 warnings.simplefilter("ignore", UnknownTimezoneWarning)
 
 # define time formats for each
 GOOGLENEWS_TF = "%m/%d/%Y"
 # TWITTER_TF = "%Y-%m-%d"
 TWITTER_TF = "%Y-%m-%dT%H:%M:%SZ"
+TWINT_TF = "%Y-%m-%d %H:%M:%S"
 
 NEWSAPI_URL = "https://newsapi.org/v2/everything?"
 TWITTER_URL = "https://api.twitter.com/2/tweets/search/recent"
@@ -313,12 +318,12 @@ class TwitterParser:
         twitter api bearer token
 
     Methods
-    .......
+    -------
     get_tweets(topic)
         takes a topic and gets the tweets for it
     """
 
-    def __init__(self, bearer_token) -> None:
+    def __init__(self, bearer_token, tqdisable=False) -> None:
         """
         Parameters
         ----------
@@ -326,6 +331,7 @@ class TwitterParser:
             twitter api bearer token
         """
         self.bearer_token = bearer_token
+        self.tqdisable = tqdisable
 
     def parse_tweet(self, tweet):
         """Parse a tweet and return just what we need
@@ -416,9 +422,103 @@ class TwitterParser:
         return tweets
 
 
-class Elijiah:
+class TwintParser:
     """
-    Performs sentiment analysis on sentences. Named after the biblical prophet
+    Gets tweets for analyzing
+
+    ...
+
+    Attributes
+    ----------
+    bearer_token : str
+        twitter api bearer token
+
+    Methods
+    -------
+    get_tweets(topic)
+        takes a topic and gets the tweets for it
+    """
+
+    def __init__(self, tqdisable=False) -> None:
+        """
+        Parameters
+        ----------
+        bearer_token : str
+            twitter api bearer token
+        """
+        self.config = twint.Config()
+        self.config.Lang = "en"
+        self.config.Pandas = True
+        self.config.Hide_output = True
+        self.tqdisable = tqdisable
+
+    def parse_tweet(self, tweet):
+        """Parse a tweet and return just what we need
+
+        Parameters
+        ----------
+        tweet : json
+            the raw tweet object
+
+        Returns
+        -------
+        data : dict
+            a dict of just the important parts (id, text, created_at)
+        """
+        data = {
+            "id": tweet["id"],
+            "created_at": tweet["created_at"],
+            "text": tweet["text"],
+        }
+        return data
+
+    def get_tweets(
+        self,
+        topic,
+        pages=2,
+        start_time=(datetime.now() - timedelta(2)).strftime(TWITTER_TF),
+        end_time=datetime.now().strftime(TWITTER_TF),
+    ):  # how many days back to go
+        """Get the tweets for a given topic
+
+        Parameters
+        ----------
+        topic : str
+            topic to search for
+        start_time : str = (datetime.now() - timedelta(2)).strftime(TWITTER_TF)
+            how far back to search from in time format %Y-%m-%dT%H:%M:%SZ'
+        end_time : str = datetime.now().strftime(TWITTER_TF)
+            how recent to search from in time format %Y-%m-%dT%H:%M:%SZ'
+
+        Returns
+        -------
+        tweets : pd.DataFrame
+            a dataframe of the tweets
+            Sample row:
+                             created_at        id                        text
+        Fri Apr 23 17:44:44 +0000 2021  138565089  RT @jenine1207: @siiyuun...
+        """
+        # define params
+
+        tweets = pd.DataFrame()
+        self.config.Search = topic
+        self.config.Limit = round(pages * 100)  # make sure its an int
+        self.config.Since = datetime.strptime(start_time, TWITTER_TF).strftime(TWINT_TF)
+        self.config.Until = datetime.strptime(end_time, TWITTER_TF).strftime(TWINT_TF)
+
+        try:
+            twint.run.Search(self.config)
+            tweets = twint.storage.panda.Tweets_df[["id", "created_at", "tweet"]]
+            tweets = tweets.rename(columns={"tweet": "text"})
+        except Exception as e:
+            warnings.warn(f"Error while getting tweets ({e})")
+        return tweets
+
+
+class Abraham:
+    """
+    Performs sentiment analysis on a search term by taking care of gathering
+    all the articles and scoring. Named after the biblical prophet
 
     ...
 
@@ -430,27 +530,113 @@ class Elijiah:
         flair model for the tweets
     lemmatizer : WordNetLemmatizer
         lemmatizer from nltk
+    news_source : str
+        where to get the news from (google or newsapi)
+    splitting : bool
+        whether or not to recursively analyze each sentence
+    weights : dict
+        how to weight the title, desc, and text attributes
+        ex: {"title": 0.2, "desc": 0.3, "text": 0.5}
+    loud : bool
+        print unnecessary output (for debugging ususally)
+    bearer_token : str
+        bearer token for the twitter api
+    tqdisable : bool
+        disale progressbars
 
     Methods
     -------
-    sentiment_analyzer_sent(sentence)
+    _sentiment_analyzer_sent(sentence)
         analyzes a single sentence and returns the score
-    normalize_text(sentence)
+    _normalize_text(sentence)
         normalizes a sentence (removes tags, lemmatizes, etc...)
-    analyze_news_text(frame, section, recursive)
+    _analyze_news_text(frame, section, recursive)
         takes a dataframe and a section and scores each row. if recursive=True, it will only
         analyze one sentence at time
+    get_articles
+        gets articles for a single search term
+    twitter_sentiment
+        takes a list of topics and gets the raw scores for each
+        (per topic per text type per row)
+    twitter_summary
+        takes a list of topics and computes the avg scores for each
+    news_summary
+        takes a list of topics and computes the avg scores for each
+    news_sentiment
+        takes a list of topics and gets the raw scores for each
+        (per topic per text type per row)
+    summary
+        takes a list of topics and gets the news and twitter summary
+        avg for each topic
+    news_summary_interval
+        takes a list of topics and gets the NEWS summary over each
+        'period' intervals from oldest to newest
+    twitter_summary_interval
+        takes a list of topics and gets the TWITTER summary over each
+        'period' intervals from oldest to newest
+    summary_interval
+        takes a list of topics and gets the NEWS+TWITTER AVG
+        summary over each 'period' intervals from oldest to newest
     """
 
-    def __init__(self, tqdisable=False) -> None:
+    def __init__(
+        self,
+        news_source="google",
+        newsapi_key=None,
+        bearer_token=None,
+        weights={"title": 0.33, "desc": 0.33, "text": 0.34},
+        loud=False,
+        tqdisable=True,
+    ) -> None:
+        """
+        Parameters
+        ----------
+        news_source : str = "google"
+            where to get the news from
+        newsapi_key : str = None
+            api key to connect to newsapi.org
+        bearer_token : str  = None
+            bearer token for the twitter api
+        spliting : bool = False
+            recursively analyze each sentence or not
+        weights : dict = {"title": 0.33, "desc": 0.33, "text": 0.34}
+            how to weight the title, desc, and text attributes within the news
+        loud : dict = False
+            print unnecessary output (for debugging ususally)
+        tqdisable : bool = True
+            disable progressbars
+        """
+
+        if news_source == "newsapi":
+            if newsapi_key:
+                self.news_source = "newsapi"
+                self.newsparser = NewsAPI(newsapi_key=newsapi_key)
+            else:
+                print(
+                    "You requested newsapi but no key was provided. Defaulting to googlenews."
+                )
+                self.news_source = "google"
+                self.newsparser = GoogleNewsParser()
+        else:
+            self.news_source = "google"
+            self.newsparser = GoogleNewsParser()
+        self.weights = weights
+        self.loud = loud
+
+        if bearer_token:
+            self.twitterparser = TwitterParser(bearer_token, tqdisable=True)
+        else:
+            self.twitterparser = TwintParser(tqdisable=True)
+
+        # sentiment analysis directly
         self.vader = SentimentIntensityAnalyzer()
         self.lemmatizer = WordNetLemmatizer()
-        print("Importing Flair")
+
         self.sunflair = flair.models.TextClassifier.load("en-sentiment")
-        nltk.download("vader_lexicon")
+        # nltk.download("vader_lexicon")
         self.tqdisable = tqdisable
 
-    def sentiment_analyzer_sent(self, sentence: str):
+    def _sentiment_analyzer_sent(self, sentence: str):
         """Analyzes a single sentence and returns the score
 
         Parameters
@@ -467,7 +653,7 @@ class Elijiah:
         print("{:-<40} {}".format(sentence, str(score)))
         return score
 
-    def normalize_text(self, sentence: str):
+    def _normalize_text(self, sentence: str):
         """Normalizes a sentence
 
         Parameters
@@ -505,7 +691,7 @@ class Elijiah:
         except:
             return sentence
 
-    def analyze_news_text(
+    def _analyze_news_text(
         self, frame: pd.DataFrame, section: str, recursive: bool = False
     ):
         """Takes a dataframe and a section and scores each row of frame[section]. If recursive=True, it will only
@@ -542,14 +728,14 @@ class Elijiah:
 
             if recursive:
                 for sentence in sent_tokenize(item):
-                    sentence = self.normalize_text(sentence)
+                    sentence = self._normalize_text(sentence)
                     score = self.vader.polarity_scores(sentence)
                     if score["compound"] != 0:  # remove all zeroes
                         score["sentence"] = sentence
                         score["datetime"] = date
                         scores.append(score)
             else:
-                sentence = self.normalize_text(item)
+                sentence = self._normalize_text(item)
                 score = self.vader.polarity_scores(sentence)
                 if score["compound"] != 0:  # remove all zeroes
                     score["sentence"] = sentence
@@ -558,7 +744,7 @@ class Elijiah:
         scores = pd.DataFrame(scores)
         return scores
 
-    def analyze_flair_text(self, tweets: pd.DataFrame, section: str):
+    def _analyze_flair_text(self, tweets: pd.DataFrame, section: str):
         """Takes a dataframe of tweets and analyzes and saves the score for each row
         ...
 
@@ -578,133 +764,41 @@ class Elijiah:
         probs = []
         sentiments = []
         newframe = tweets.copy()
-        newframe[section] = newframe[section].apply(self.normalize_text)
-        for tweet in tqdm(
-            newframe[section].to_list(),
-            desc=section,
-            leave=False,
-            dynamic_ncols=True,
-            disable=self.tqdisable,
-        ):
-            # if tweet empty
-            if len(tweet) == 0:
-                probs.append("NEUTRAL")
-                sentiments.append("NEUTRAL")
-            else:
-                # make prediction
-                sentence = flair.data.Sentence(tweet)
-                self.sunflair.predict(sentence)
-                # extract sentiment prediction
-                probs.append(sentence.labels[0].score)  # numerical score 0-1
-                sentiments.append(sentence.labels[0].value)  # 'POSITIVE' or 'NEGATIVE'
+        try:
+            newframe[section] = newframe[section].apply(self._normalize_text)
 
-        # add probability and sentiment predictions to tweets dataframe
+            for tweet in tqdm(
+                newframe[section].to_list(),
+                desc=section,
+                leave=False,
+                dynamic_ncols=True,
+                disable=self.tqdisable,
+            ):
+                # if tweet empty
+                if not tweet:
+                    tweet = ""  # make string if none
+                if len(tweet) == 0:
+                    probs.append("NEUTRAL")
+                    sentiments.append("NEUTRAL")
+                else:
+                    # make prediction
+                    sentence = flair.data.Sentence(tweet)
+                    self.sunflair.predict(sentence)
+                    # extract sentiment prediction
+                    probs.append(sentence.labels[0].score)  # numerical score 0-1
+                    sentiments.append(
+                        sentence.labels[0].value
+                    )  # 'POSITIVE' or 'NEGATIVE'
+        except:
+            probs = ["NEUTRAL"] * newframe.shape[0]
+            sentiments = ["NEUTRAL"] * newframe.shape[0]
+            # add probability and sentiment predictions to tweets dataframe
         newframe["probability"] = probs
         newframe["sentiment"] = sentiments
         newframe = newframe[newframe.probability != "NEUTRAL"]
         newframe = newframe[newframe.sentiment != "NEUTRAL"]
         return newframe
 
-
-class Isaiah:
-    """
-    Performs sentiment analysis on a search term by taking care of gathering
-    all the articles and scoring. Named after the biblical prophet
-
-    ...
-
-    Attributes
-    ----------
-    sia : Elijiah
-        Elijiah analyzer
-    news_source : str
-        where to get the news from (google or newsapi)
-    splitting : bool
-        whether or not to recursively analyze each sentence
-    weights : dict
-        how to weight the title, desc, and text attributes
-        ex: {"title": 0.2, "desc": 0.3, "text": 0.5}
-    loud : bool
-        print unnecessary output (for debugging ususally)
-    bearer_token : str
-        bearer token for the twitter api
-    tqdisable : bool
-        disale progressbars
-
-    Methods
-    -------
-    get_articles
-        gets articles for a single search term
-    twitter_sentiment
-        takes a list of topics and gets the raw scores for each
-        (per topic per text type per row)
-    twitter_summary
-        takes a list of topics and computes the avg scores for each
-    news_summary
-        takes a list of topics and computes the avg scores for each
-    news_sentiment
-        takes a list of topics and gets the raw scores for each
-        (per topic per text type per row)
-    summary
-        takes a list of topics and gets the news and twitter summary
-        avg for each topic
-    news_summary_interval
-        takes a list of topics and gets the NEWS summary over each
-        'period' intervals from oldest to newest
-    twitter_summary_interval
-        takes a list of topics and gets the TWITTER summary over each
-        'period' intervals from oldest to newest
-    summary_interval
-        takes a list of topics and gets the NEWS+TWITTER AVG
-        summary over each 'period' intervals from oldest to newest
-    """
-
-    def __init__(
-        self,
-        news_source="google",
-        newsapi_key=None,
-        bearer_token=None,
-        weights={"title": 0.33, "desc": 0.33, "text": 0.34},
-        loud=False,
-        tqdisable=False,
-    ) -> None:
-        """
-        Parameters
-        ----------
-        news_source : str = "google"
-            where to get the news from
-        newsapi_key : str = None
-            api key to connect to newsapi.org
-        bearer_token : str  = None
-            bearer token for the twitter api
-        spliting : bool = False
-            recursively analyze each sentence or not
-        weights : dict = {"title": 0.33, "desc": 0.33, "text": 0.34}
-            how to weight the title, desc, and text attributes
-        loud : dict = False
-            print unnecessary output (for debugging ususally)
-        tqdisable : bool = False
-            disale progressbars
-        """
-        self.sia = Elijiah(tqdisable=True)
-        if news_source == "newsapi":
-            if newsapi_key:
-                self.news_source = "newsapi"
-                self.newsparser = NewsAPI(newsapi_key=newsapi_key)
-            else:
-                print(
-                    "You requested newsapi but no key was provided. Defaulting to googlenews."
-                )
-                self.news_source = "google"
-                self.newsparser = GoogleNewsParser()
-        else:
-            self.news_source = "google"
-            self.newsparser = GoogleNewsParser()
-        self.weights = weights
-        self.loud = loud
-        self.bearer_token = bearer_token
-        self.tqdisable = tqdisable
-
     def get_articles(
         self,
         topics: list,
@@ -861,13 +955,13 @@ class Isaiah:
         articles = self.get_articles(topics, start_time=start_time, end_time=end_time)
         scores = {}
         for topic in articles:
-            titles = self.sia.analyze_flair_text(
+            titles = self._analyze_flair_text(
                 articles[topic][["title", "datetime"]], "title"
             )
-            desc = self.sia.analyze_flair_text(
+            desc = self._analyze_flair_text(
                 articles[topic][["desc", "datetime"]], "desc"
             )
-            text = self.sia.analyze_flair_text(
+            text = self._analyze_flair_text(
                 articles[topic][["text", "datetime"]], "text"
             )
             scores[topic] = {"title": titles, "desc": desc, "text": text}
@@ -902,24 +996,27 @@ class Isaiah:
             topics, size=size, start_time=start_time, end_time=end_time
         )
         for topic in raws:
-            scores[topic] = (
-                round(
-                    raws[topic]
-                    .loc[raws[topic]["sentiment"] == "POSITIVE"]
-                    .dropna()
-                    .shape[0]
-                    * (100 / raws[topic].shape[0]),
-                    1,
-                ),
-                round(
-                    raws[topic]
-                    .loc[raws[topic]["sentiment"] == "NEGATIVE"]
-                    .dropna()
-                    .shape[0]
-                    * (100 / raws[topic].shape[0]),
-                    1,
-                ),
-            )
+            try:
+                scores[topic] = (
+                    round(
+                        raws[topic]
+                        .loc[raws[topic]["sentiment"] == "POSITIVE"]
+                        .dropna()
+                        .shape[0]
+                        * (100 / raws[topic].shape[0]),
+                        1,
+                    ),
+                    round(
+                        raws[topic]
+                        .loc[raws[topic]["sentiment"] == "NEGATIVE"]
+                        .dropna()
+                        .shape[0]
+                        * (100 / raws[topic].shape[0]),
+                        1,
+                    ),
+                )
+            except:
+                scores[topic] = (-1, -1)
         return scores
 
     def twitter_sentiment(
@@ -947,17 +1044,12 @@ class Isaiah:
         scores : dict
             a dict of dataframe of scores for each topic
         """
-        if not self.bearer_token:
-            warnings.warn("No bearer token provided on instantiation.")
-            return {}
-
         scores = {}
-        twitterparser = TwitterParser(self.bearer_token)
         for topic in topics:
-            tweets = twitterparser.get_tweets(
+            tweets = self.twitterparser.get_tweets(
                 topic, pages=int(size / 100), start_time=start_time, end_time=end_time
             )
-            scored_frame = self.sia.analyze_flair_text(tweets, "text")
+            scored_frame = self._analyze_flair_text(tweets, "text")
             scores[topic] = scored_frame
         return scores
 
@@ -1050,7 +1142,9 @@ class Isaiah:
         topics,
         oldest=datetime.now() - timedelta(days=1),
         newest=datetime.now(),
-        period=timedelta(hours=1),
+        interval=timedelta(hours=12),
+        offset=timedelta(hours=1),
+        size=100,
     ):
         """Get the NEWS summary over each 'period' intervals from oldest to newest
 
@@ -1062,8 +1156,12 @@ class Isaiah:
             oldest datetime to search from
         newest : datetime.datetime
             newest datetime to search up to
-        period : timedelta
+        interval : timedelta
+            interval to grab the data with
+        offset : timedelta
             interval to advance through with
+        size : int = 100
+            roughly how many articles to get per interval
 
         Returns
         -------
@@ -1074,32 +1172,31 @@ class Isaiah:
 
         for topic in topics:
             now = newest
-            df = pd.DataFrame(
-                columns=["start_time", "end_time", "positive", "negative"]
-            )
+            df = pd.DataFrame(columns=["timestamp", "positive", "negative"])
 
             for i in trange(
-                self._intervals(oldest, newest, period),
+                self._intervals(oldest, newest, offset),
                 leave=True,
                 dynamic_ncols=True,
                 desc="backtest",
                 disable=self.tqdisable,
             ):
-                pre = now - period
+                pre = now - offset
 
                 scores = self.news_summary(
                     [topic],
-                    start_time=pre.strftime(TWITTER_TF),
+                    size=size,
+                    start_time=(now - interval).strftime(TWITTER_TF),
                     end_time=now.strftime(TWITTER_TF),
                 )[topic]
 
                 # add to dataframe
                 df = df.append(
                     {
-                        "start_time": pre,
-                        "end_time": now,
+                        "timestamp": now,
                         "positive": scores[0],
                         "negative": scores[1],
+                        "lag": interval,
                     },
                     ignore_index=True,
                 )
@@ -1113,7 +1210,9 @@ class Isaiah:
         topics,
         oldest=datetime.now() - timedelta(days=1),
         newest=datetime.now(),
-        period=timedelta(hours=1),
+        interval=timedelta(hours=12),
+        offset=timedelta(hours=1),
+        size=100,
     ):
         """Get the TWITTER summary over each 'period' intervals from oldest to newest
 
@@ -1125,8 +1224,12 @@ class Isaiah:
             oldest datetime to search from
         newest : datetime.datetime
             newest datetime to search up to
-        period : timedelta
+        interval : timedelta
+            interval to grab the data with
+        offset : timedelta
             interval to advance through with
+        size : int = 100
+            roughly how many tweets to get per interval
 
         Returns
         -------
@@ -1137,32 +1240,32 @@ class Isaiah:
 
         for topic in topics:
             now = newest
-            df = pd.DataFrame(
-                columns=["start_time", "end_time", "positive", "negative"]
-            )
+            df = pd.DataFrame(columns=["timestamp", "positive", "negative"])
 
             for i in trange(
-                self._intervals(oldest, newest, period),
+                self._intervals(oldest, newest, offset),
                 leave=True,
                 dynamic_ncols=True,
                 desc="backtest",
                 disable=self.tqdisable,
             ):
-                pre = now - period
+                pre = now - offset
+                farthest = now - interval
 
                 scores = self.twitter_summary(
                     [topic],
-                    start_time=pre.strftime(TWITTER_TF),
+                    size=size,
+                    start_time=farthest.strftime(TWITTER_TF),
                     end_time=now.strftime(TWITTER_TF),
                 )[topic]
 
                 # add to dataframe
                 df = df.append(
                     {
-                        "start_time": pre,
-                        "end_time": now,
+                        "timestamp": now,
                         "positive": scores[0],
                         "negative": scores[1],
+                        "lag": interval,
                     },
                     ignore_index=True,
                 )
@@ -1200,9 +1303,7 @@ class Isaiah:
 
         for topic in topics:
             now = newest
-            df = pd.DataFrame(
-                columns=["start_time", "end_time", "positive", "negative"]
-            )
+            df = pd.DataFrame(columns=["timestamp", "positive", "negative"])
 
             for i in trange(
                 self._intervals(oldest, newest, period),
@@ -1222,10 +1323,10 @@ class Isaiah:
                 # add to dataframe
                 df = df.append(
                     {
-                        "start_time": pre,
-                        "end_time": now,
+                        "timestamp": now,
                         "positive": scores[0],
                         "negative": scores[1],
+                        "lag": period,
                     },
                     ignore_index=True,
                 )
